/*
 * This function will list the contents that we have stored with tsmpipe
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "dsmrc.h"
#include "dsmapitd.h"
#include "../tsmpipe.h"

dsBool_t headerPrinted=bFalse; // Have we rendered the heading row yet

/*
 * Our Callback to show what is in TSM
 */
int tsm_listfile_cb(dsmQueryType qType, DataBlk *qResp, void *userdata) {
	char stor[2];
	char state[3];
	char ced[4];
	char objInfo[DSM_MAX_OBJINFO_LENGTH];

	if (userdata != NULL ) {
		fprintf(stderr, "tsm_listfile_cb: Internal error: userdata != NULL");
		return -1;
	}

	if (qType == qtArchive) {
		if (! headerPrinted++) {
			printf("%40s %2s %s %3s %19s %19s %10s %10s %10s",
				"NAME",
				"ST",
				"L",
				"CED",
				"BACKUP",
				"EXPIRE",
				"SIZE",
				"ID",
				"DESC"
			);

#ifdef USE_DIGEST
			printf(" %s","OBJINFO");
#endif
			printf("\n");
		}

		qryRespArchiveData respArchive;
		memset(&respArchive,0x00,sizeof(qryRespArchiveData));
		qryRespArchiveData *qr = (void *) qResp->bufferPtr;
		respArchive = *qr;

		// The Object Status
		switch (respArchive.objName.objType) {
			case (DSM_OBJ_FILE) : strcpy(state,"F"); break;
			case (DSM_OBJ_DIRECTORY) : strcpy(state,"D"); break;
			default: strcpy(state,"?");
		}

		// Location
		switch (respArchive.mediaClass) {
			case (MEDIA_FIXED) : strcpy(stor,"D"); break;
			case (MEDIA_LIBRARY) : strcpy(stor,"T"); break;
			default: strcpy(stor,"?");
		}

		// Compression, Encryption, De-Duplication
		strcpy(ced,(respArchive.compressType == DSM_OBJ_COMPRESSED_YES ? "C" :
			(respArchive.compressType == DSM_OBJ_COMPRESSED_NO ? "-" : "?")));

		strcat(ced,(respArchive.encryptionType & DSM_ENCRYPT_CLIENTENCRKEY ? "C" :
			(respArchive.encryptionType & DSM_ENCRYPT_USER ? "U" : "-")));

		strcat(ced,respArchive.clientDeduplicated ? "D" : "-");

		// The Object Status
		printf("%40s|%2s|%s|%3s|%19s|%19s|%7.3f MB|%u-%u|%s",dsmObjnameToStr(respArchive.objName),state,stor,ced,dsmDateToStr(respArchive.insDate),respArchive.expDate.year ? dsmDateToStr(respArchive.expDate) : "",dsmSizeToNum(respArchive.sizeEstimate,bTrue),respArchive.objId.hi,respArchive.objId.lo,respArchive.descr);

	} else if (qType == qtBackup) {
		if (! headerPrinted++) {
			printf("%40s %2s %s %3s %19s %19s %10s %10s",
				"NAME",
				"ST",
				"L",
				"CED",
				"BACKUP",
				"EXPIRE",
				"SIZE",
				"ID"
			);

#ifdef USE_DIGEST
			printf(" %s","OBJINFO");
#endif
			printf("\n");
		}

		qryRespBackupData respBackup;

		memset(&respBackup,0x00,sizeof(qryRespBackupData));
		qryRespBackupData *qr = (void *) qResp->bufferPtr;
		respBackup = *qr;

		// The Object Status
		switch (respBackup.objName.objType) {
			case (DSM_OBJ_FILE) : strcpy(state,"F"); break;
			case (DSM_OBJ_DIRECTORY) : strcpy(state,"D"); break;
			default: strcpy(state,"?");
		}

		switch (respBackup.objState) {
			case (DSM_ACTIVE) : strcat(state,"A"); break;
			case (DSM_INACTIVE) : strcat(state,"I"); break;
			default: strcat(state,"?");
		}

		// Location
		switch (respBackup.mediaClass) {
			case (MEDIA_FIXED) : strcpy(stor,"D"); break;
			case (MEDIA_LIBRARY) : strcpy(stor,"T"); break;
			default: strcpy(stor,"?");
		}

		// Compression, Encryption, De-Duplication
		strcpy(ced,(respBackup.compressType == DSM_OBJ_COMPRESSED_YES ? "C" :
			(respBackup.compressType == DSM_OBJ_COMPRESSED_NO ? "-" : "?")));

		strcat(ced,(respBackup.encryptionType & DSM_ENCRYPT_CLIENTENCRKEY ? "C" :
			(respBackup.encryptionType & DSM_ENCRYPT_USER ? "U" : "-")));

		strcat(ced,respBackup.clientDeduplicated ? "D" : "-");

#ifdef USE_DIGEST
		// Object Info
		memset(&objInfo,0x00,sizeof(objInfo));

		if (respBackup.objInfolen)
			strncat(objInfo,respBackup.objInfo,respBackup.objInfolen);
#endif

		printf("%40s|%2s|%s|%3s|%19s|%19s|%7.3f MB|%u-%8u",dsmObjnameToStr(respBackup.objName),state,stor,ced,dsmDateToStr(respBackup.insDate),respBackup.expDate.year ? dsmDateToStr(respBackup.expDate) : "",dsmSizeToNum(respBackup.sizeEstimate,bTrue),respBackup.objId.hi,respBackup.objId.lo);
	} else {
		fprintf(stderr,"tsm_listfile_cb: Internal error: Unknown qType %d\n",qType);
		return -1;
	}

#ifdef USE_DIGEST
	printf("|%s",objInfo);
#endif
	printf("\n");

	return 1;
}

/*
 * List objects that are in TSM
 */
int tsm_listfile(dsUint32_t sesshandle, dsmQueryType qType, qryArchiveData qaData, qryBackupData qbData) {
	dsInt16_t rc=0;

	rc = tsm_queryfile(sesshandle,qType,tsm_listfile_cb,NULL,qaData,qbData);

	if (rc != DSM_RC_OK && rc != DSM_RC_ABORT_NO_MATCH)
		return 0;

	return 1;
}
